@@ -1,14 +1,23 @@
-from typing import Any, Awaitable, Callable, List, NoReturn, Optional, Tuple, TypeVar, overload
+from typing import (
+    Any,
+    Awaitable,
+    Callable,
+    Iterable,
+    List,
+    NoReturn,
+    Optional,
+    TypeVar,
+    cast,
+)
 
 from expression.collections import seq
 from expression.core import (
     MailboxProcessor,
     Option,
-    TailCallResult,
     TailCall,
+    TailCallResult,
     aiotools,
     compose,
-    fst,
     match,
     pipe,
     tailrec_async,
@@ -18,15 +27,21 @@ from expression.system.disposable import AsyncDisposable
 from .combine import zip_seq
 from .notification import Notification, OnCompleted, OnError, OnNext
 from .observables import AsyncAnonymousObservable
-from .observers import AsyncAnonymousObserver, AsyncNotificationObserver, auto_detach_observer
+from .observers import (
+    AsyncAnonymousObserver,
+    AsyncNotificationObserver,
+    auto_detach_observer,
+)
 from .transform import map, transform
-from .types import AsyncObservable, AsyncObserver, Stream
+from .types import AsyncObservable, AsyncObserver, Filter, Projection
 
 TSource = TypeVar("TSource")
 TResult = TypeVar("TResult")
 
 
-def choose_async(chooser: Callable[[TSource], Awaitable[Option[TResult]]]) -> Stream[TSource, TResult]:
+def choose_async(
+    chooser: Callable[[TSource], Awaitable[Option[TResult]]]
+) -> Projection[TSource, TResult]:
     async def handler(next: Callable[[TResult], Awaitable[None]], xs: TSource) -> None:
         result = await chooser(xs)
         for x in result.to_list():
@@ -35,8 +50,12 @@ def choose_async(chooser: Callable[[TSource], Awaitable[Option[TResult]]]) -> St
     return transform(handler)
 
 
-def choose(chooser: Callable[[TSource], Option[TResult]]) -> Stream[TSource, TResult]:
-    def handler(next: Callable[[TResult], Awaitable[None]], xs: TSource) -> Awaitable[None]:
+def choose(
+    chooser: Callable[[TSource], Option[TResult]]
+) -> Projection[TSource, TResult]:
+    def handler(
+        next: Callable[[TResult], Awaitable[None]], xs: TSource
+    ) -> Awaitable[None]:
         for x in chooser(xs).to_list():
             return next(x)
         return aiotools.empty()
@@ -44,7 +63,9 @@ def choose(chooser: Callable[[TSource], Option[TResult]]) -> Stream[TSource, TRe
     return transform(handler)
 
 
-def filter_async(predicate: Callable[[TSource], Awaitable[bool]]) -> Stream[TSource, TSource]:
+def filter_async(
+    predicate: Callable[[TSource], Awaitable[bool]]
+) -> Projection[TSource, TSource]:
     """Filter async.
 
     Filters the elements of an observable sequence based on an async
@@ -58,15 +79,17 @@ def filter_async(predicate: Callable[[TSource], Awaitable[bool]]) -> Stream[TSou
         Stream[TSource, TSource]: [description]
     """
 
-    async def handler(next: Callable[[TSource], Awaitable[None]], x: TSource):
+    async def handler(next: Callable[[TSource], Awaitable[None]], x: TSource) -> None:
         if await predicate(x):
             return await next(x)
 
     return transform(handler)
 
 
-def filter(predicate: Callable[[TSource], bool]) -> Stream[TSource, TSource]:
-    def handler(next: Callable[[TSource], Awaitable[None]], x: TSource) -> Awaitable[None]:
+def filter(predicate: Callable[[TSource], bool]) -> Projection[TSource, TSource]:
+    def handler(
+        next: Callable[[TSource], Awaitable[None]], x: TSource
+    ) -> Awaitable[None]:
         if predicate(x):
             return next(x)
         return aiotools.empty()
@@ -74,12 +97,9 @@ def filter(predicate: Callable[[TSource], bool]) -> Stream[TSource, TSource]:
     return transform(handler)
 
 
-@overload
-def starfilter(predicate: Callable[[TSource, int], bool]) -> Stream[Tuple[TSource, int], Tuple[TSource, int]]:
-    ...
-
-
-def starfilter(predicate: Callable[..., bool]) -> Stream[Tuple[Any, ...], Tuple[Any, ...]]:
+def starfilter(
+    predicate: Callable[..., bool]
+) -> Projection[Iterable[Any], Iterable[Any]]:
     """Filter and spread the arguments to the predicate.
 
     Filters the elements of an observable sequence based on a predicate.
@@ -88,7 +108,9 @@ def starfilter(predicate: Callable[..., bool]) -> Stream[Tuple[Any, ...], Tuple[
         sequence that satisfy the condition.
     """
 
-    def handler(next: Callable[[Tuple[TSource, ...]], Awaitable[None]], args: Tuple[TSource, ...]) -> Awaitable[None]:
+    def handler(
+        next: Callable[[Iterable[Any]], Awaitable[None]], args: Iterable[Any]
+    ) -> Awaitable[None]:
         if predicate(*args):
             return next(args)
         return aiotools.empty()
@@ -96,15 +118,22 @@ def starfilter(predicate: Callable[..., bool]) -> Stream[Tuple[Any, ...], Tuple[
     return transform(handler)
 
 
-def filteri(predicate: Callable[[TSource, int], bool]) -> Stream[TSource, TSource]:
-    return compose(
-        zip_seq(seq.infinite()),
+def filteri(
+    predicate: Callable[[TSource, int], bool]
+) -> Callable[
+    [AsyncObservable[TSource]], AsyncObservable[TSource]
+]:  # Projection[TSource, TSource]:
+    ret = compose(
+        zip_seq(seq.infinite),
         starfilter(predicate),
-        map(fst),
+        map(seq.head),
     )
+    return cast(Projection[TSource, TSource], ret)  # FIXME: pyright issue?
 
 
-def distinct_until_changed(source: AsyncObservable[TSource]) -> AsyncObservable[TSource]:
+def distinct_until_changed(
+    source: AsyncObservable[TSource],
+) -> AsyncObservable[TSource]:
     """Distinct until changed.
 
     Return an observable sequence only containing the distinct
@@ -122,12 +151,14 @@ def distinct_until_changed(source: AsyncObservable[TSource]) -> AsyncObservable[
 
         async def worker(inbox: MailboxProcessor[Notification[TSource]]) -> None:
             @tailrec_async
-            async def message_loop(latest: Notification[TSource]) -> TailCallResult[NoReturn]:
+            async def message_loop(
+                latest: Notification[TSource],
+            ) -> TailCallResult[NoReturn]:
                 n = await inbox.receive()
 
                 async def get_latest() -> Notification[TSource]:
-                    with match(n) as m:
-                        for x in OnNext.case(m):
+                    with match(n) as case:
+                        for x in case(OnNext[TSource]):
                             if n == latest:
                                 break
                             try:
@@ -135,10 +166,10 @@ def distinct_until_changed(source: AsyncObservable[TSource]) -> AsyncObservable[
                             except Exception as ex:
                                 await safe_obv.athrow(ex)
                             break
-                        for err in OnError.case(m):
+                        for err in case(OnError[TSource]):
                             await safe_obv.athrow(err)
                             break
-                        while m.case(OnCompleted):
+                        while case(OnCompleted):
                             await safe_obv.aclose()
                             break
 
@@ -147,7 +178,9 @@ def distinct_until_changed(source: AsyncObservable[TSource]) -> AsyncObservable[
                 latest = await get_latest()
                 return TailCall(latest)
 
-            await message_loop(OnCompleted)  # Use as sentinel value as it will not match any OnNext value
+            await message_loop(
+                OnCompleted
+            )  # Use as sentinel value as it will not match any OnNext value
 
         agent = MailboxProcessor.start(worker)
 
@@ -160,7 +193,7 @@ def distinct_until_changed(source: AsyncObservable[TSource]) -> AsyncObservable[
     return AsyncAnonymousObservable(subscribe_async)
 
 
-def skip(count: int) -> Stream[TSource, TSource]:
+def skip(count: int) -> Filter:
     """[summary]
 
     Bypasses a specified number of elements in an observable sequence
@@ -194,29 +227,12 @@ def skip(count: int) -> Stream[TSource, TSource]:
     return _skip
 
 
-def skip_last(count: int) -> Stream[TSource, TSource]:
+def skip_last(count: int) -> Filter:
     def _skip_last(source: AsyncObservable[TSource]) -> AsyncObservable[TSource]:
-        """Bypasses a specified number of elements at the end of an
-        observable sequence.
-
-        This operator accumulates a queue with a length enough to store
-        the first `count` elements. As more elements are received,
-        elements are taken from the front of the queue and produced on
-        the result sequence. This causes elements to be delayed.
-
-        Args:
-            count: Number of elements to bypass at the end of the
-            source sequence.
-
-        Returns:
-            An observable sequence containing the source sequence
-            elements except for the bypassed ones at the end.
-        """
-
         async def subscribe_async(observer: AsyncObserver[TSource]) -> AsyncDisposable:
             safe_obv, auto_detach = auto_detach_observer(observer)
 
-            q = []
+            q: List[TSource] = []
 
             async def asend(value: TSource) -> None:
                 front = None
@@ -235,7 +251,7 @@ def skip_last(count: int) -> Stream[TSource, TSource]:
     return _skip_last
 
 
-def take(count: int) -> Stream[TSource, TSource]:
+def take(count: int) -> Filter:
     if count < 0:
         raise ValueError("Count cannot be negative.")
 
@@ -262,7 +278,7 @@ def take(count: int) -> Stream[TSource, TSource]:
     return _take
 
 
-def take_last(count: int) -> Stream[TSource, TSource]:
+def take_last(count: int) -> Filter:
     """Take last elements from stream.
 
     Returns a specified number of contiguous elements from the end of an
@@ -298,7 +314,7 @@ def take_last(count: int) -> Stream[TSource, TSource]:
     return _take_last
 
 
-def take_until(other: AsyncObservable[TResult]) -> Stream[TSource, TSource]:
+def take_until(other: AsyncObservable[Any]) -> Filter:
     """Take elements until other.
 
     Returns the values from the source observable sequence until the
@@ -329,7 +345,9 @@ def take_until(other: AsyncObservable[TResult]) -> Stream[TSource, TSource]:
     return _take_until
 
 
-def slice(start: Optional[int] = None, stop: Optional[int] = None, step: int = 1) -> Stream[TSource, TSource]:
+def slice(
+    start: Optional[int] = None, stop: Optional[int] = None, step: int = 1
+) -> Filter:
     """Slices the given source stream.
 
     It is basically a wrapper around skip(), skip_last(), take(),
@@ -378,7 +396,8 @@ def slice(start: Optional[int] = None, stop: Optional[int] = None, step: int = 1
         if step is not None:
             if step > 1:
                 mapper: Callable[[Any, int], bool] = lambda _, i: i % step == 0
-                source = pipe(source, filteri(mapper))
+                xs = pipe(source, filteri(mapper))
+                source = xs
             elif step < 0:
                 # Reversing streams is not supported
                 raise TypeError("Negative step not supported.")
