@@ -7,4 +7,10 @@ from .subject import AsyncTestSingleSubject, AsyncTestSubject
 from .utils import ca
 from .virtual_events import VirtualTimeEventLoop
 
-__all__ = ["ca", "VirtualTimeEventLoop", "AsyncTestObserver", "AsyncTestSingleSubject", "AsyncTestSubject"]
+__all__ = [
+    "ca",
+    "VirtualTimeEventLoop",
+    "AsyncTestObserver",
+    "AsyncTestSingleSubject",
+    "AsyncTestSubject",
+]
