@@ -10,26 +10,21 @@ log = logging.getLogger(__name__)
 
 def noop(*args: Any, **kw: Any) -> None:
     """No operation. Returns nothing"""
-    pass
 
 
-async def anoop(value: Optional[TSource] = None):
+async def anoop(value: Optional[Any] = None) -> None:
     """Async no operation. Returns nothing"""
-    pass
 
 
 class NoopObserver(AsyncObserver[TSource]):
     async def asend(self, value: TSource) -> None:
         log.debug("NoopSink:asend(%s)", str(value))
-        pass
 
     async def athrow(self, error: Exception) -> None:
         log.debug("NoopSink:athrow(%s)", error)
-        pass
 
     async def aclose(self) -> None:
         log.debug("NoopSink:aclose()")
-        pass
 
 
-noopobserver = NoopObserver()
+noopobserver: NoopObserver[Any] = NoopObserver()
